/*  DeltaSMT: Delta-debugger for Satisfiablity Modulo Theories (SMT) benchmarks.
 *  Copyright (C) 2009  Robert Daniel Brummayer
 *
 *  This file is part of DeltaSMT.
 *
 *  DeltaSMT is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DeltaSMT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class Extrapred extends SMTNode
{

  protected String name;

  public Extrapred (String name, SMTNode []children){
    super(SMTNodeKind.EXTRAPRED, SMTTypeManager.booleanType, children);
    assert (name != null);
    assert (children == null || children.length > 0);
    this.name = name;
  }

  public String toDeclString(){
    StringBuilder builder = new StringBuilder();
    builder.append (this.name);
    if (super.children != null) {
      for (SMTNode child : super.children) {
        builder.append (" ");
        builder.append (child.type.toString());
      }
    }
    return builder.toString();
  }

  public String hashKey () {
    StringBuilder builder = new StringBuilder();
    builder.append (name);
    if (children != null) {
      for (SMTNode child : children) {
        builder.append (" ");
        builder.append (child.getId());
      }
    }
    return builder.toString();
  }

  public String toString(){
    return this.name;
  }

  public String getName(){
    return this.name;
  }

  boolean isVar (){
    return super.children == null;
  }

}
