/*  DeltaSMT: Delta-debugger for Satisfiablity Modulo Theories (SMT) benchmarks.
 *  Copyright (C) 2009  Robert Daniel Brummayer
 *
 *  This file is part of DeltaSMT.
 *
 *  DeltaSMT is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DeltaSMT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class SMTNode implements Comparable<SMTNode>
{

  protected static long g_id = 0L;


  protected SMTNodeKind kind;

  protected SMTType type;

  protected int refs;

  protected long id;

  protected SMTNode children[];

  protected byte mark;


  public SMTNode (SMTNodeKind kind, SMTType type){
    assert (kind != null);
    assert (type != null);

    this.kind = kind;
    this.type = type;
    this.refs = 1;
    this.id = ++g_id;
    this.children = null;
    this.mark = 0;
  }

  public SMTNode (SMTNodeKind kind, SMTType type, SMTNode n0){
    this (kind, type);

    assert (kind.getArity() == 1);
    assert (n0 != null);

    this.children = new SMTNode[1];
    this.children[0] = n0;
    n0.incRefCounter();
  }

  public SMTNode (SMTNodeKind kind, SMTType type, SMTNode n0, SMTNode n1){
    this (kind, type);

    assert (kind.getArity() == 2);
    assert (n0 != null);
    assert (n1 != null);

    this.children = new SMTNode[2];
    this.children[0] = n0;
    this.children[1] = n1;
    n0.incRefCounter();
    n1.incRefCounter();
  }

  public SMTNode (SMTNodeKind kind, SMTType type, SMTNode n0, SMTNode n1, 
                  SMTNode n2){
    this (kind, type);

    assert (kind.getArity() == 3);
    assert (n0 != null);
    assert (n1 != null);
    assert (n2 != null);

    this.children = new SMTNode[3];
    this.children[0] = n0;
    this.children[1] = n1;
    this.children[2] = n2;
    n0.incRefCounter();
    n1.incRefCounter();
    n2.incRefCounter();
  }

  public SMTNode (SMTNodeKind kind, SMTType t, SMTNode[] children) {
    this (kind, t);
    if (children != null && children.length > 0) {
      this.children = new SMTNode[children.length];
      for (int i = 0; i < children.length; i++) {
        assert (children[i] != null);
        this.children[i] = children[i];
        this.children[i].incRefCounter();
      }
    }
  }


  public void incRefCounter(){
    this.refs++;
  }

  public void DecRefCounter(){
    assert (this.refs > 0);
    this.refs--;
  }

  public SMTType getType(){
    return this.type;
  }

  public void setType(SMTType type){
    this.type = type;
  }

  public int getRefs(){
    return this.refs;
  }

  public SMTNodeKind getKind(){
    return this.kind;
  }

  public long getId(){
    return this.id;
  }

  public SMTNode[] getChildren(){
    return this.children;
  }

  public byte getMark (){
    return this.mark;
  }

  public void setMark (byte mark){
    this.mark = mark;
  }

  public SMTNode copy() {
    this.refs++;
    return this;
  }

  public int compareTo (SMTNode n){
    assert (n != null);

    if (this.id < n.id)
      return -1;
    if (this.id > n.id)
      return 1;
    
    return 0;
  }

  public String toString(){
    return this.kind.getString();
  }

  public String hashKey(){
    StringBuilder builder = new StringBuilder();
    builder.append (this.kind.toString());
    assert (this.children != null);
    for (SMTNode child : children){
      builder.append(" ");
      builder.append(child.id);
    }
    return builder.toString();
  }

}
