/*  DeltaSMT: Delta-debugger for Satisfiablity Modulo Theories (SMT) benchmarks.
 *  Copyright (C) 2009  Robert Daniel Brummayer
 *
 *  This file is part of DeltaSMT.
 *
 *  DeltaSMT is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DeltaSMT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public enum SMTNodeKind { 

  /* input */
  CONST("constant", 0, false),
  /* boolean */
  TRUE("true", 0, false),
  FALSE("false", 0, false),
  NOT("not", 1, false), 
  AND("and", -1, true),
  OR("or", -1, true),
  IMPLIES("implies", 2, false),
  XOR("xor", 2, true),
  IFF("iff", 2, true),
  IF_THEN_ELSE("if_then_else", 3, false), // if-then-else on boolean operands 
  /* bit-vector operators: */
  /* unary operators */
  BVNOT("bvnot", 1, false),
  BVNEG("bvneg", 1, false),
  /* unary operators with index */
  EXTRACT("extract", 1, false),
  REPEAT("repeat", 1, false),
  ZERO_EXTEND("zero_extend", 1, false),
  SIGN_EXTEND("sign_extend", 1, false),
  ROTATE_LEFT("rotate_left", 1, false),
  ROTATE_RIGHT("rotate_right", 1, false),
  /* commutative binary operators */
  BVAND("bvand", 2, true),
  BVNAND("bvnand", 2, true),
  BVOR("bvor", 2, true),
  BVNOR("bvnor", 2, true),
  BVXOR("bvxor", 2, true),
  BVXNOR("bvxnor", 2, true),
  BVADD("bvadd", 2, true),
  BVMUL("bvmul", 2, true),
  BVCOMP("bvcomp", 2, true),
  /* non-commutative binary operators */
  BVULT("bvult", 2, false),
  BVULE("bvule", 2, false),
  BVUGT("bvugt", 2, false),
  BVUGE("bvuge", 2, false),
  BVSLT("bvslt", 2, false),
  BVSLE("bvsle", 2, false),
  BVSGT("bvsgt", 2, false),
  BVSGE("bvsge", 2, false),
  BVSHL("bvshl", 2, false),
  BVLSHR("bvlshr", 2, false),
  BVASHR("bvashr", 2, false),
  BVSUB("bvsub", 2, false),
  BVUDIV("bvudiv", 2, false),
  BVUREM("bvurem", 2, false),
  BVSDIV("bvsdiv", 2, false),
  BVSREM("bvsrem", 2, false),
  BVSMOD("bvsmod", 2, false),
  CONCAT("concat", 2, false),
  /* interpreted predicates */
  LT("<", 2, false),
  GT(">", 2, false),
  LE("<=", 2, false),
  GE(">=", 2, false),
  EQ("=", 2, true),
  DISTINCT("distinct", -1, true),
  /* interpreted functions */
  ADD("+", -1, true),
  NEG("~", 1, false),
  SUB("-", 2, false),
  MUL("*", 2, true),
  DIV("/", 2, false),
  /* array operators */
  SELECT("select", 2, false),
  STORE("store", 3, false),
  /* if-then-else on terms */
  ITE("ite", 3, false),
  EXTRAFUN ("extrafun", -1, false),
  EXTRAPRED ("extrapred", -1, false);

  protected String string;

  protected int arity; /* -1 is used to indicate n-ary operators */
  
  protected boolean commutative;


  SMTNodeKind (String string, int arity, boolean commutative){
    this.string = string;
    this.arity = arity;
    this.commutative = commutative;
  }

  public String getString(){
    return this.string;
  }

  public int getArity() {
    return this.arity;
  }

  public boolean isCommutative() {
    return this.commutative;
  }

  public static boolean isIndexedUnaryKind (SMTNodeKind kind) {
    assert (kind != null);
    return kind == SMTNodeKind.EXTRACT || kind == SMTNodeKind.REPEAT ||
           kind == SMTNodeKind.ZERO_EXTEND || kind == SMTNodeKind.SIGN_EXTEND ||
           kind == SMTNodeKind.ROTATE_LEFT || kind == SMTNodeKind.ROTATE_RIGHT;
  }

}
